/**
 * @file gui_seggerlibrary.c
 * @date 2015-10-09
 *
 * NOTE:
 * This file is generated by DAVE. Any manual modification done to this file will be lost when the code is regenerated.
 */
/**
 * @cond
 ***********************************************************************************************************************
 * GUI_SEGGERLIBRARY v4.1.10 - Displays the text or graphics on OLED/LCD
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * To improve the quality of the software, users are encouraged to share modifications, enhancements or bug fixes
 * with Infineon Technologies AG (dave@infineon.com).
 ***********************************************************************************************************************
 *
 * Change History
 * --------------
 *
 * 2015-02-16:
 *     - Initial version<br>
 *
 * 2015-03-20:
 *     - LCD APP routines are added <br>
 *
 * 2015-10-09:
 *     - CMSIS_RTOS_RTX support added
 * @endcond
 *
 */

/***********************************************************************************************************************
 * HEADER FILES
 **********************************************************************************************************************/
#include <xmc_common.h>
#include "gui_seggerlibrary.h"
#if (CMSIS_RTOS_RTX_PRESENT == 1)
#include "CMSIS_RTOS_RTX/cmsis_rtos_rtx.h"
#endif
/***********************************************************************************************************************
 * MACROS
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * LOCAL DATA
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * LOCAL ROUTINES
 **********************************************************************************************************************/
#if (SPI_INTERFACE == 1)
void GUI_SEGGERLIBRARY_SendByte(uint8_t data);
uint8_t GUI_SEGGERLIBRARY_ReadByte(void);
#endif
/**********************************************************************************************************************
* API IMPLEMENTATION
**********************************************************************************************************************/

/*
 * API to retrieve the version of the GUI_SEGGERLIBRARY APP
 */
DAVE_APP_VERSION_t GUI_SEGGERLIBRARY_GetAppVersion(void)
{
  DAVE_APP_VERSION_t version;

  version.major = GUI_SEGGERLIBRARY_MAJOR_VERSION;
  version.minor = GUI_SEGGERLIBRARY_MINOR_VERSION;
  version.patch = GUI_SEGGERLIBRARY_PATCH_VERSION;

  return (version);
}


/* API initializes the Segger library */
GUI_SEGGERLIBRARY_STATUS_t GUI_SEGGERLIBRARY_Init(GUI_SEGGERLIBRARY_t *handle)
{
  GUI_SEGGERLIBRARY_STATUS_t status;

  status = GUI_SEGGERLIBRARY_STATUS_SUCCESS;

#if (GUI_SEGGERLIBRARY_RTOS_PRESENT == 0)
  status = (GUI_SEGGERLIBRARY_STATUS_t)SYSTIMER_Init(handle->systimer_handler);
#endif

#if (CMSIS_RTOS_RTX_PRESENT == 1)
  status = (GUI_SEGGERLIBRARY_STATUS_t)CMSIS_RTOS_RTX_Init(&CMSIS_RTOS_RTX_0);
#endif

  if (status == GUI_SEGGERLIBRARY_STATUS_SUCCESS)
  {
    /* Initialize SPI as well as the port and pin for GPIO */
    #if (SPI_INTERFACE == 1)
      status = (GUI_SEGGERLIBRARY_STATUS_t)SPI_MASTER_Init(handle->spi_master_handler);

      #if !USER_DEFINED_LCD
		if (status == GUI_SEGGERLIBRARY_STATUS_SUCCESS)
		{
		  XMC_GPIO_Init(handle->config->portNo, handle->config->pinNo, handle->config->gpio);

		  status = GUI_SEGGERLIBRARY_STATUS_SUCCESS;
		}
		else
		{
		  status = GUI_SEGGERLIBRARY_STATUS_FAILURE;
		}
      #endif
    #endif

    if (handle->enable_at_init == true)
    {
      GUI_Init();
    }
  }
  else
  {
    status = GUI_SEGGERLIBRARY_STATUS_FAILURE;
  }

  return (status);
}

#if (SPI_INTERFACE == 1)

/* Sends one byte of data to the OLED through SPI */
void GUI_SEGGERLIBRARY_SendByte(uint8_t data)
{
  SPI_MASTER_ClearFlag(&SPI_INSTANCE, XMC_SPI_CH_STATUS_FLAG_TRANSMIT_SHIFT_INDICATION);

  XMC_SPI_CH_Transmit(SPI_INSTANCE.channel, (uint16_t)data, XMC_SPI_CH_MODE_STANDARD);

  while(SPI_MASTER_GetFlagStatus(&SPI_INSTANCE, XMC_SPI_CH_STATUS_FLAG_TRANSMIT_SHIFT_INDICATION)!=
       XMC_SPI_CH_STATUS_FLAG_TRANSMIT_SHIFT_INDICATION);
}

/* Reads one byte of data from the OLED through SPI */
uint8_t GUI_SEGGERLIBRARY_ReadByte(void)
{
  uint16_t lastword = 0;

  SPI_MASTER_ClearFlag(&SPI_INSTANCE,XMC_SPI_CH_STATUS_FLAG_RECEIVE_INDICATION |
		              XMC_SPI_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION);

  XMC_SPI_CH_Receive(SPI_INSTANCE.channel, XMC_SPI_CH_MODE_STANDARD);

  while(SPI_MASTER_GetFlagStatus(&SPI_INSTANCE, XMC_SPI_CH_STATUS_FLAG_RECEIVE_INDICATION |
	                            XMC_SPI_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION)!=
	                            (XMC_SPI_CH_STATUS_FLAG_RECEIVE_INDICATION |
	                            XMC_SPI_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION));

  lastword = XMC_SPI_CH_GetReceivedData(SPI_INSTANCE.channel);

  return (lastword);
}

#if (DISPLAY_8BPP == 1)
/*
 *  This routine writes to display controller using SPI bus, with A0 = 0
 *
 */
void GUI_SEGGERLIBRARY_WriteCmdByte(uint8_t cmd)
{
#if (!USER_DEFINED_LCD && SPI_INTERFACE == 1)
    /* Clear the A0 pin */
  XMC_GPIO_SetOutputLow(IOHandle.portNo,IOHandle.pinNo);
#endif

  XMC_SPI_CH_EnableSlaveSelect(SPI_INSTANCE.channel,SPI_INSTANCE.config->slave_select_pin_config[0]->slave_select_ch);

  /* Write to the SPI Bus */
  GUI_SEGGERLIBRARY_SendByte(cmd);

  XMC_SPI_CH_DisableSlaveSelect(SPI_INSTANCE.channel);
}

/*
 *  This routine writes to display controller using SPI bus, with A0 = 1
 *
 */
void GUI_SEGGERLIBRARY_WriteDataByte(uint8_t data)
{
#if (!USER_DEFINED_LCD && SPI_INTERFACE == 1)
    /* Set the A0 pin*/
  XMC_GPIO_SetOutputHigh(IOHandle.portNo,IOHandle.pinNo);
#endif

  XMC_SPI_CH_EnableSlaveSelect(SPI_INSTANCE.channel, SPI_INSTANCE.config->slave_select_pin_config[0]->slave_select_ch);

  GUI_SEGGERLIBRARY_SendByte(data);

  XMC_SPI_CH_DisableSlaveSelect(SPI_INSTANCE.channel);
}

/*
 *  This routine writes multiple bytes to display controller using SPI bus,
 *  with A0 = 1.
 *
 */
void GUI_SEGGERLIBRARY_WriteMultipleBytes(uint8_t *data, int numitems)
{
  uint8_t datacopy;

#if (!USER_DEFINED_LCD && SPI_INTERFACE == 1)
  /* Set the A0 pin*/
  XMC_GPIO_SetOutputHigh(IOHandle.portNo,IOHandle.pinNo);
#endif

  XMC_SPI_CH_EnableSlaveSelect(SPI_INSTANCE.channel, SPI_INSTANCE.config->slave_select_pin_config[0]->slave_select_ch);

  do
  {
    datacopy = *data;
    GUI_SEGGERLIBRARY_SendByte(datacopy);
	data++;
  }while(--numitems);

  XMC_SPI_CH_DisableSlaveSelect(SPI_INSTANCE.channel);
}

/*
 *  This routine read multiple bytes from display controller using SPI bus,
 *  with A0 = 1.
 *
 */
void GUI_SEGGERLIBRARY_ReadMultipleBytes(uint8_t *data, int numitems)
{
#if (!USER_DEFINED_LCD && SPI_INTERFACE == 1)
  XMC_GPIO_SetOutputHigh(IOHandle.portNo,IOHandle.pinNo);
#endif

  SPI_MASTER_Receive(&SPI_INSTANCE, data, numitems);
}
#endif

#endif
